'''
Created on Jul 25, 2013

@author: marten
'''
from django.utils.translation import ugettext, ugettext_lazy as _
from basesoft.web.core.config import DefaultConfig
from pvcycle.core.models import PvcTbsLicense
from basesoft.web.contrib.ticketing.config import TicketingLicenseConfig
from basesoft.web.core.input.validation import plugins

class PvcTbsLicenseConfigs(DefaultConfig):
    permissions = ('pvc_tbs_license_configs', )
    label = _('TBS settings')
    is_expanded = True

    def configs(self):
        return {
            'pvc.tbs.declaration.period': {
                'label': ugettext('Declaration period'),
                'options': {
                    'options': {
                        PvcTbsLicense.DECLARATION_PERIOD_MONTHLY: ugettext('Monthly'),
                        PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY: ugettext('Quarterly'),
                        PvcTbsLicense.DECLARATION_PERIOD_YEARLY: ugettext('Yearly'),
                    }
                }
            },
            'pvc.tbs.declaration.closing.day': {
                'label': ugettext('Declaration closing day')
            },
            'pvc.tbs.fee.day': {
                'label': ugettext(u'Day in January on which the Administrative/Membership fee is triggered for this tbs')
            },
            'pvc.tbs.has.full.memberships': { # members with voting rights
                'label': ugettext('Full membership available')
            },
            'pvc.tbs.declaration.use.weight.for.amount.calculations': {
                'label': ugettext('Declaration uses weight for amount calculations')
            },
            'pvc.tbs.was.active.in.2013': {
                'label': ugettext('Membership was possible in 2013')
            },
            # Included from IQPortal order app
            'basesoft.web.contrib.order.minimum_invoiceable_amount': {
                'label': ugettext('Minimum invoiceable amount')
            },
            'pvc.tbs.minimum.contribution.fee': {
                'label': ugettext('Minimum contribution fee for non-compliant members')
            },
            'pvc.tbs.use.membership.instead.of.service.level': {
                'label': ugettext('Use Membership levels instead of Service levels')
            },
            'pvc.tbs.membership.valid.for.years': {
                'label': ugettext('Memberships are valid for how many years')
            },
            'pvc.tbs.auto.approve.member': {
                'label': ugettext('Members applications are automatically approved')
            },
            'pvc.tbs.require.company.registration.number': {
                'label': ugettext('Application of a member requires a Company Registration Number')
            },
            'pvc.tbs.require.accepting.of.terms.and.conditions' : {
                'label': ugettext('Members requires accepting of Terms and Conditions on application')
            },
            'pvc.tbs.require.member.in.own.country' : {
                'label': ugettext('Member company requires to be in the same country as the TBS')
            },
            'pvc.tbs.optional.uk.vat.no' : {
                'label': ugettext('VAT number for UK members is optional at this TBS')
            }
        }

    def get_form_behaviour(self, validators, values):

        closing_day_integer_params = {'unsigned': True, 'allow_zero': False}
        if not values.has_key('pvc.tbs.declaration.period') or values['pvc.tbs.declaration.period'] == None:
            validators['pvc.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]
            return validators

        p = int(values['pvc.tbs.declaration.period'])
        if p == PvcTbsLicense.DECLARATION_PERIOD_MONTHLY:
            closing_day_integer_params['max_value'] = 27 # february

        if p == PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY:
            closing_day_integer_params['max_value'] = 89 # quarter 1

        if p == PvcTbsLicense.DECLARATION_PERIOD_YEARLY:
            closing_day_integer_params['max_value'] = 364

        validators['pvc.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]

        return validators


license_configs = {
    'PvcTbsLicense' : (PvcTbsLicenseConfigs(), ),
    'PvcOperatorLicense' : (TicketingLicenseConfig(), )
}
