'''
Created on Jul 16, 2013

@author: marten
'''
from django.utils.translation import ugettext_lazy as _
from sqlalchemy import func, and_

from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.file.models import File
from sqlalchemy.orm.session import Session
from basesoft.util.virtual_date import today

class PvcOperatorLicense(License):
    """
    @DynamicAttrs
    """
    NAME_I18N = _("PVC Operator")
    REQUIRED_PERMISSIONS = ['pvc_operator_license']

    def __get_controller(self):
        from pvcycle.core.controllers import PvcOperatorController
        return PvcOperatorController

    controller = property(__get_controller)

    def __repr__(self):
        return '<PvcOperatorLicense id=%s>' % self.id


class PvcAccountantLicense(License):
    """
    @DynamicAttrs
    """
    NAME_I18N = _('PVC Accountant')
    REQUIRED_PERMISSIONS = ['pvc_accountant_license']

    def __repr__(self):
        return '<PvcAccountantLicense id=%s>' % self.id


class PvcTbsLicense(License):
    """
    @DynamicAttrs
    """
    NAME_I18N = _('PVC Take Back Scheme')
    REQUIRED_PERMISSIONS = ['pvc_tbs_license']

    TBS_TYPE_SUBSIDIARY = 1 # implies invoicing.
    TBS_TYPE_LOCAL = 2 # only aggregated data import.

    DECLARATION_PERIOD_MONTHLY = 1
    DECLARATION_PERIOD_YEARLY = 2
    DECLARATION_PERIOD_QUARTERLY = 3

    def __repr__(self):
        return '<PvcTbsLicense id=%s>' % self.id

class PvcTbsServiceLevel(object):
    """
    @DynamicAttrs
    """

    def get_current_administration_fee(self):
        session = Session.object_session(self)
        return session.query(func.pvcGetCurrentAdministrationFee(self.id, today().year)).one()[0]

    def get_rounded_administration_fee(self):
        from decimal import Decimal
        return self.get_current_administration_fee().quantize(Decimal('.01'))

    current_admin_fee = property(get_rounded_administration_fee)

    def get_current_contribution_fee(self):
        session = Session.object_session(self)
        return session.query(func.pvcGetCurrentContributionFee(self.id, today())).one()[0]

    def get_rounded_contribution_fee(self):
        from decimal import Decimal
        return self.get_current_contribution_fee().quantize(Decimal('.01'))

    current_contrib_fee = property(get_rounded_contribution_fee)

    def get_current_fee_start_year(self):
        return today().year

    def __repr__(self):
        return '<PvcTbsServiceLevel id="%s">' % self.id

class PvcTbsServiceLevelAdministrationFee(object):
    """
    @DynamicAttrs
    """
    pass

class PvcTbsServiceLevelContributionFee(object):
    """
    @DynamicAttrs
    """
    pass

class PvcAccountantTbs(object):
    """
    @DynamicAttrs
    """
    pass

class PvcMemberLicense(License):
    """
    @DynamicAttrs
    """
    NAME_I18N = _('PVC Member License')
    REQUIRED_PERMISSIONS = ['pvc_tbs_member_manager'] # this can also be created from PUBLIC or by Operator?.

    def __get_controller(self):
        from pvcycle.core.controllers import PvcMemberController
        return PvcMemberController

    controller = property(__get_controller)

    def __repr__(self):
        return '<PvcMemberLicense id=%s>' % self.id


class PvcTbsMember(object):
    """
    @DynamicAttrs
    """
    STATUS_INCOMPLETE = 0
    STATUS_PENDING = 1
    STATUS_APPROVED = 2
    STATUS_REJECTED = 3
    STATUS_SUSPENDED = 4

    STATUS_RESIGNING = 5
    STATUS_RESIGNED = 6

    def _get_active_service_level(self):
        session = Session.object_session(self)
        return session.query(PvcTbsMemberServiceLevel).filter(
            and_(
                PvcTbsMemberServiceLevel.tbs_member==self,
                PvcTbsMemberServiceLevel.start_date<=today()
            )
        ).order_by(PvcTbsMemberServiceLevel.start_date.desc()).first()

    active_service_level = property(_get_active_service_level)

    def _get_active_membership(self):
        session = Session.object_session(self)
        return session.query(PvcTbsMemberMembership).filter(
            and_(
                PvcTbsMemberMembership.tbs_member==self,
                PvcTbsMemberMembership.start_date<=today()
            )
        ).order_by(PvcTbsMemberMembership.start_date.desc()).first()

    active_membership = property(_get_active_membership)

    def __repr__(self):
        return '<PvcTbsMember id=%s>' % self.id

class PvcTbsMemberServiceLevel(object):
    """
    @DynamicAttrs
    """
    BACKREF = 'service_level'

class PvcTbsMemberContractFile(File):
    """
    @DynamicAttrs
    """

    TYPE_CONTRACT_FULL = 1
    TYPE_CONTRACT_ANNEX = 2

    def __repr__(self):
        return '<PvcTbsMemberContractFile id=%s>' % self.id


class PvcTbsCertificate(object):
    """
    @DynamicAttrs
    """

class PvcTbsMemberCertificate(object):
    """
    @DynamicAttrs
    """

    STATUS_PAID = 0
    STATUS_APPROVED = 1
    STATUS_REJECTED = 2
    STATUS_REVOKED = 3


    def __get_issue_date(self):
        return self.created.date()

    issue_date = property(__get_issue_date)

class PvcTbsMembership(object):
    """
    @DynamicAttrs
    """
    def get_current_fee(self):
        session = Session.object_session(self)
        return session.query(func.pvcGetCurrentMembershipFee(self.id, today().year)).one()[0]

    # compatibility with PvcTbsServiceLevel
    get_current_administration_fee = get_current_fee

    def get_current_fee_start_year(self):
        session = Session.object_session(self)
        return session.query(func.pvcGetCurrentMembershipStartYear(self.id, today().year)).one()[0]

    def get_rounded_fee(self):
        from decimal import Decimal
        return self.get_current_fee().quantize(Decimal('.01'))

    current_admin_fee = property(get_rounded_fee)

    def __repr__(self):
        return '<PvcTbsMembership id="%s">' % self.id

class PvcTbsMembershipFee(object):
    """
    @DynamicAttrs
    """
    pass

class PvcTbsMemberMembership(object):
    """
    @DynamicAttrs
    """
    BACKREF = 'membership'

class PvcROECountry(object):
    """
    @DynamicAttrs
    """
    pass
