'''
Created on Oct 10, 2013

@author: marten
'''
from django.utils.translation import ugettext_lazy, ugettext as _

from sqlalchemy.sql.expression import and_
from sqlalchemy.orm.util import aliased
from sqlalchemy import func

from basesoft.web.contrib.order.custom_query import AbstractOrderCustomQuery, Merchant, AbstractPaymentMethodCustomQuery
from basesoft.web.core.util.custom_query import CustomQueryField, CustomQueryModel
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.order.models import InvoiceFile, PaymentMethod, Currency

from pvcycle.order.models import PvcDeclarationOrder
from pvcycle.core.controllers import AbstractPvcController
from pvcycle.core.models import PvcTbsMember
from pvcycle.core.custom_query import AbstractPvcCustomQuery

Commissioner = aliased(License)

class OrderCustomQuery(AbstractPvcCustomQuery, AbstractOrderCustomQuery):

    def _get_member_column(self):
        return License.id

    I18N_CHECKOUT_LABEL = ugettext_lazy('Submit')
    permissions = ['pvc_view_orders']

    def tables(self):
        return self._tables()

    def fields(self):
        return self._fields()

class DeclarationCustomQuery(AbstractPvcCustomQuery, AbstractOrderCustomQuery):

    I18N_CHECKOUT_LABEL = ugettext_lazy('Submit')
    permissions = ['pvc_view_declarations']

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def tables(self):
        return (
            CustomQueryModel(PvcDeclarationOrder),
            CustomQueryModel(PvcTbsMember, onclause=PvcDeclarationOrder.producer_id==PvcTbsMember.id),
            CustomQueryModel(License, onclause=PvcTbsMember.member_id==License.id),
            CustomQueryModel(Commissioner, onclause=PvcDeclarationOrder.client_id==Commissioner.id),
            CustomQueryModel(Merchant, onclause=PvcDeclarationOrder.merchant_id==Merchant.id),
            CustomQueryModel(Currency, onclause=PvcDeclarationOrder.currency_id==Currency.id),
            CustomQueryModel(InvoiceFile, onclause=PvcDeclarationOrder.invoice_id==InvoiceFile.file_id, use_outerjoin=True, priv=None),
        )

    def fields(self):
        return (
            CustomQueryField(PvcDeclarationOrder.id, 'Order ID', key='id', is_filter=True),
            CustomQueryField(PvcDeclarationOrder.polymorphic_type, 'Polymorphic type', key='polymorphic_type'),
            CustomQueryField(PvcDeclarationOrder.id, _('Details'), key='details'),
            CustomQueryField(PvcDeclarationOrder.year, _('Year'), key='year', is_filter=True, filter_options=PvcDeclarationOrder.year),
            CustomQueryField(PvcDeclarationOrder.month, _('Month'), key='month', is_filter=True, filter_options=PvcDeclarationOrder.month),
            CustomQueryField(PvcDeclarationOrder.quarter, _('Quarter'), key='quarter', is_filter=True, filter_options=PvcDeclarationOrder.quarter),
            CustomQueryField(PvcDeclarationOrder.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcDeclarationOrder.STATUS_NEW,
                'label': _("New"),
            },{
                'value': PvcDeclarationOrder.STATUS_PENDING,
                'label': _("Declared"),
            },{
                'value': PvcDeclarationOrder.STATUS_SUBMITTED,
                'label': _("Submitted"),
            },{
                'value': PvcDeclarationOrder.STATUS_CREDITED,
                'label': _("Credited"),
            }]),
            CustomQueryField(PvcDeclarationOrder.is_compliant, _('Compliant'), key='is_compliant', is_filter=True, filter_options=[{
                'value': PvcDeclarationOrder.IS_COMPLIANT,
                'label': _('Compliant')
            },{
                'value': PvcDeclarationOrder.IS_NON_COMPLIANT,
                'label': _('Non-compliant')
            }]),
            CustomQueryField(PvcDeclarationOrder.type, _('Type'), key='type', is_filter=True),
            CustomQueryField(PvcDeclarationOrder.resubmit_request, _('Resubmit request'), key='resubmit_request', is_filter=True),
            CustomQueryField(PvcDeclarationOrder.export_date, _('Export date'), key='export_date'),
            CustomQueryField(License.name, _('Producer'), key='client', is_filter=True),
            CustomQueryField(Commissioner.name, _('Commissioner'), key='commissioner', is_filter=True),
            CustomQueryField(Merchant.name, _('TBS'), key='merchant', is_filter=True, is_group_by=True),
            CustomQueryField(func.ifnull(Currency.symbol, Currency.code), _('Currency'), 'currency'),
            CustomQueryField(PvcDeclarationOrder.total_amount_excluding_vat, _('Total amount (ex VAT)'), 'total_amount'),
            CustomQueryField(PvcDeclarationOrder.total_amount_including_vat, _('Total amount (incl VAT)'), 'total_amount_incl_vat'),
            CustomQueryField(PvcDeclarationOrder.status, _('Action'), key='action'), # depending on the status the action text is different
        )

    group_by = (PvcDeclarationOrder.id, )

class OutstandingDeclarationCustomQuery(DeclarationCustomQuery):
    def set_where(self):
        super(OutstandingDeclarationCustomQuery, self).set_where()
        self.where = and_(
            self.where,
            PvcDeclarationOrder.status.in_([PvcDeclarationOrder.STATUS_NEW, PvcDeclarationOrder.STATUS_PENDING]),
            PvcDeclarationOrder.type==PvcDeclarationOrder.TYPE_DEBIT
        )

class PaymentMethodCustomQuery(AbstractPaymentMethodCustomQuery, AbstractPvcController):
    def set_where(self):
        super(PaymentMethodCustomQuery, self).set_where()
        self.where = and_(self.where, PaymentMethod.merchant_id.in_([x.id for x in self._get_take_back_schemes(permissions=['manage_payment_methods'])]))

custom_queries = (OrderCustomQuery, DeclarationCustomQuery, OutstandingDeclarationCustomQuery, PaymentMethodCustomQuery, )
