# -*- coding: utf-8 -*-

# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from pvcycle.core.icons import AbstractPvcIcon

class PvcOrderManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-order-manager'
    title = _('Order manager')
    icon_index = 72
    component = 'pvc_order_manager'
    group = 'financial'

    char_main = u'\uE805'
    char_sub = u'\uE806'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_view_orders')
        self._assert_pvc_license(controller)

class PvcDeclarationManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-declaration-manager'
    title = _('Declaration manager')
    href = '#'
    icon_index = 71
    component = 'pvc_declaration_manager'
    group = 'financial'

    char_main = u'\uE802'
    char_sub = u'\uE806'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_view_declarations')
        self._assert_pvc_license(controller)

class PvcOutstandingDeclarationsIcon(AbstractPvcIcon):

    id = 'icon-pvc-outstanding-declarations'
    title = _('Outstanding declarations')
    href = '#'
    icon_index = 70
    component = 'pvc_outstanding_declarations'
    group = 'financial'

    # number icon (unread notifications)
    number_type = 'important'
    number_rpc_url = 'pvcycle.order.DeclarationHandler'
    number_rpc_method = 'get_open_declaration_count'

    char_main = u'\uE802'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_view_declarations', 'pvc_member_license') # only for members
        self._assert_pvc_license(controller)

class PvcPaymentMethodsIcon(AbstractPvcIcon):
    id = 'icon-pvc-payment-methods'
    title = _('Payment method manager')
    href = '#'
    icon_index = 78
    component = 'pvc_payment_method_manager'
    group = 'financial'

    char_main = u'\uE835' # euro sign
    char_sub = u'\uE804' # data table

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_payment_methods') # only for members
        self._assert_pvc_license(controller)

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [PvcOrderManagerIcon(), PvcDeclarationManagerIcon(), PvcOutstandingDeclarationsIcon(), PvcPaymentMethodsIcon()]
