'''
Created on Oct 8, 2013

@author: marten
'''
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from basesoft.web.contrib.order.models import Order, OrderLine
from basesoft.web.contrib.mailing.models import Mail
    
class PvcOrder(Order):
    """
    @DynamicAttrs
    """
    auto_submit = True
    permissions = ('pvc_view_orders', )
    js_details_module = 'pvc_order_details'

    name_i18n = _('Administrative')

    def get_name_i18n(self):
        return ugettext('Administrative')

    def get_order_controller(self):
        from pvcycle.order.controllers import PvcOrderController
        return PvcOrderController

class PvcOrderLineAdministrationFee(OrderLine):
    """
    This is used for membership registration
    @DynamicAttrs
    """
    tax_class_code = 'TAX_ADMIN_FEE'
    gl_account_code = 'GL_ADMIN_FEE'

    BACKREF = 'tbs_member_service_level'

class PvcOrderLineMembershipFee(OrderLine):
    """
    This is used for membership registration in the UK
    @DynamicAttrs
    """
    tax_class_code = 'TAX_MEMBER_FEE'
    gl_account_code = 'GL_MEMBER_FEE'

    BACKREF = 'tbs_member_membership'

class PvcDeclarationOrder(Order):
    """
    @DynamicAttrs
    """
    RESUBMIT_REQUEST_PENDING = 1
    RESUBMIT_REQUEST_APPROVED = 2

    IS_NON_COMPLIANT = 0
    IS_COMPLIANT = 1

    auto_submit = False
    permissions = ('pvc_view_declarations', )
    js_details_module = 'pvc_declaration_details'

    name_i18n = _('Contribution')

    all_privileges = (Order.PRIV_VIEW, Order.PRIV_UPDATE, Order.PRIV_ARCHIVE, Order.PRIV_SUBMIT, Order.PRIV_PAY)
    producer_privileges = (Order.PRIV_VIEW, Order.PRIV_UPDATE, Order.PRIV_ARCHIVE, Order.PRIV_SUBMIT) # given to the Producer of the modules
    tbs_member_privileges = (Order.PRIV_VIEW, Order.PRIV_PAY) # Given to the license who pays for it, either same as Producer or Commissioner

    def get_order_controller(self):
        from pvcycle.order.controllers import PvcDeclarationOrderController
        return PvcDeclarationOrderController

    def get_name_i18n(self):
        return ugettext('Contribution')

class PvcDeclarationOrderLine(OrderLine):
    """
    @DynamicAttrs
    """
    tax_class_code = 'TAX_CONTRIB_FEE'
    gl_account_code = 'GL_CONTRIB_FEE'
    MAX_LINE_QUANTITY = 1000 * 1000 # Ticket #319

class PvcDeclarationOrderLineMinimumCF(OrderLine):
    """
    @DynamicAttrs
    """
    tax_class_code = 'TAX_FORFEIT_FEE'
    gl_account_code = 'GL_FORFEIT_FEE'
    MAX_LINE_QUANTITY = 1000 * 1000 # Ticket #319


class PvcDeclarationReminderMail(Mail):
    PRIV_USE_FOR_SENDING = 64

    NAME_I18N = _("Declaration reminder mail")

    def get_privileges(self):
        return Mail.get_privileges(self) + [
            (self.PRIV_USE_FOR_SENDING, _('Use for sending'))
        ]

    @staticmethod
    def replacement_fields():
        x = Mail.replacement_fields()
        x.update({
            ugettext('Company'): {
                'company.name': ugettext('Name'),
            },
            ugettext('Declaration'): {
                'declaration.year': ugettext('Year'),
                'declaration.quarter': ugettext('Quarter'),
                'declaration.month': ugettext('Month'),
            },
        })
        return x

    def _replace_field(self, person, field_name, context):

        tokens = field_name.split('.')

        obj = None

        if tokens[0] == 'company':
            obj = context['license']
            assert tokens[1] in ['name']

        if tokens[0] == 'declaration':
            obj = context['declaration']
            assert tokens[1] in ['year', 'month', 'quarter']

        if obj is not None:
            x = getattr(obj, tokens[1])
            if x is None: x = '-'
            return unicode(x)

        return Mail._replace_field(self, person, field_name, context)
