'''
Created on Nov 5, 2013

@author: marten
'''
from django.utils.translation import ugettext as _

from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.core.exceptions import PublicUserException

from pvcycle.order.controllers import PvcDeclarationOrderController
from pvcycle.order.models import PvcDeclarationOrder, PvcDeclarationOrderLine
from pvcycle.core.controllers import PvcMemberController, PvcTBSController
from pvcycle.pvmodule.models import PvcTbsMemberPVModule

class DeclarationHandler:

    def __init__(self, request, **kwargs):

        self._request = request
        self._sa_session = request.Session()
        self._account_id = request.session['user_id']
        self._controller = PvcDeclarationOrderController(self._sa_session, account_id=self._account_id, request=request)

    def get_open_declaration_count(self):
        rv = len(self._controller.get_open_orders(priv=PvcDeclarationOrder.PRIV_UPDATE))
        if rv > 0:
            return rv
        return None

    def close_declarations_before_specified_date(self, tbs_id, before):
        controller = PvcTBSController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs = controller.get_take_back_schemes(tbs_id).pop()
        declarations = self._controller.close_declarations_created_before_specified_date(tbs, before)
        return len(declarations)

    def validate_additional_lines(self, args):
        v = {}
        for k in args.keys():
            if k[:19] == 'quantity_additional':
                v[k] = [(plugins.Integer, {'unsigned': True, 'allow_zero': True, 'max_value': PvcDeclarationOrderLine.MAX_LINE_QUANTITY}), (plugins.Required, {})]

        form_handler = FormValidator(v)
        return form_handler.validate(args)

    def validate_declaration(self, args):
        v = {}
        for k in args.keys():
            v[k] = [(plugins.Integer, {'unsigned': True, 'allow_zero': True, 'max_value': PvcDeclarationOrderLine.MAX_LINE_QUANTITY}), (plugins.Required, {})]

        form_handler = FormValidator(v)
        return form_handler.validate(args)

    def validate_manual_declaration(self, args):
        form_handler = FormValidator({
            'tbs_id': [(plugins.Required, {}), ]
        })
        return form_handler.validate(args)

    def load_declaration(self, _id):
        return self._controller.load_state(_id, True)

    def _save_declaration(self, _id, args):
        subrogations = None
        additional = None
        if args.has_key('subrogation'):
            subrogations = {}
            for k, v in args['subrogation'].iteritems():
                if v == '':
                    continue
                subrogations[k[12:]] = int(v)
        if args.has_key('additional'):
            additional = args['additional']
        declaration = args
        if args.has_key('declaration'):
            declaration = args['declaration']
        return self._controller.update_declaration(_id, declaration, additional_lines=additional, subrogate=subrogations)

    def save_declaration(self, _id, args):
        order = self._save_declaration(_id, args)
        if type(order) != list:
            return order.id
        return _id

    def save_declaration_and_submit(self, _id, args):
        order = self._save_declaration(_id, args)
        if type(order) != list:
            self._controller.submit_order(order.id)
            return order.id
        [self._controller.submit_order(o.id) for o in order]
        return _id

    def checkout_declaration(self, _id):
        order = self._controller.submit_order(_id)
        return order.id

    def get_new_lines_for_declaration(self, _id, module_id):
        return self._controller.get_new_lines(_id, module_id)

    def get_undeclared_periods(self, tbs_id, member_id=None):
        # TODO: fix for multi-member stuff.
        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id) if tm.tbs.id==int(tbs_id)].pop()
        if len(tbs_member.modules) == 0:
            raise PublicUserException(_('No PV Modules selected at this TBS.'))
        return self._controller.get_undeclared_periods(tbs_member)

    def save_manual_declaration(self, bitbucket, params):
        member_id = None
        tbs_id = params['tbs_member']['tbs_id']
        del params['period_select']['warning']
        # TODO: fix for multi-member stuff.
        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id) if tm.tbs.id==int(tbs_id)].pop()
        rv = self._controller.add_open_declaration(tbs_member, **params['period_select'])
        if rv is None:
            raise PublicUserException(_('No declaration could be triggered for supplied period, are PV Modules selected?'))
        if isinstance(rv, PvcDeclarationOrder):
            return rv.id
        return [d.id for d in rv]

    def remove_line(self, _id, line_id, remove_for_future=False):
        remove_for_future = PvcTbsMemberPVModule.IS_INACTIVE if remove_for_future else None
        order = self._controller.remove_line(_id, line_id, remove_for_future)
        return order.id

    def send_declaration_reminder_email(self, declaration_ids):
        self._controller.send_declaration_reminder_email(declaration_ids)

    def request_for_resubmission(self, declaration_id):
        self._controller.request_for_resubmission(declaration_id)
        return True

    def approve_resubmission(self, declaration_id):
        self._controller.approve_resubmit_request(declaration_id)
        return True

    def reject_resubmission(self, declaration_id, reason):
        self._controller.reject_resubmit_request(declaration_id, reason)
        return True

handlers = (DeclarationHandler,)
